/* uPAC-7186EXD-CAN:Use 7186EXD to send SDO to control 2024C,										

										(1)Change AO value

										(2)Change type code										

										uPAC-7186EXD-CAN + CAN-2024C										

		node ID :0x01		

		Baud Rate Rotary Switch: 3 (125 kbps)    

    Compiler: TC++ 3.0

    Compile mode: large

    Project: SDO.c

	     7186el.Lib

	     xc100l.Lib             

    Communication: CANBus of 7186EXD <==>CANopen<==>CANBus of 2054C																							

    Hardware: uPAC-7186EXD-CAN + CAN-2054C     

    SDO Communication Entries :

								Idx  	Sidx  	Description  											Type  			Attr  Default

								1200h  0h  		largest sub-index supported for   UNSIGNED 8   RO  	 2

															"server SDO parameter"

											 1h     COB-ID form client to server 			UNSIGNED 32  RO  	 600h+x	

											 				(RxSDO)

											 2h     COB-ID form server to client 	    UNSIGNED 32  RO    580h+x

															(TxSDO) 															

    Analogue Output Function :

								Idx 		Sidx  Description 							  Type 				 Attr Default 

								6411h   0h		Number of Output 8-Bit  	  UNSIGNED 16	 RO   1

												1h    AO value of channel 0   	  UNSIGNED 16  RW   0

												2h		AO value of channel 1			  UNSIGNED 16  RW   0

												3h    AO value of channel 2   	  UNSIGNED 16  RW   0

												4h    AO value of channel 3   	  UNSIGNED 16  RW   0												

    						2004h   0h    Number of Analogue Outputs  UNSIGNED 8   RO   4

												1h    Type Code of AO Channel 0   UNSIGNED 8   RW   0x33

												2h    Type Code of AO Channel 1   UNSIGNED 8   RW   0x33

												3h    Type Code of AO Channel 2   UNSIGNED 8   RW   0x33

												4h    Type Code of AO Channel 3   UNSIGNED 8   RW   0x33									

		Type Code :		

								0x30~0x31 for Current

								0x32~0x35 for Voltage		

								Default : 0x33 (-10V DC ~ +10V DC) (HEX:0x8000 ~ 0x7FFF)

[07,Dec,2011] by Alan

*/

#include <stdlib.h>

#include "..\..\lib\7186e.h"

#include "..\..\lib\xc100.h"

unsigned char MsgData[8];
unsigned int PrintFlag = 1;
unsigned long MsgCnt=0;

/*Receive CAN message */

void ReceiveCANData(void);

/*Send SDO (COB-ID form client to server (RxSDO),COB-ID:600h + node ID)*/

void SendCANSDO(unsigned char cmd,unsigned char index_l,unsigned char index_h,unsigned char sub_index,

						    unsigned char D1,unsigned char D2,unsigned char D3,unsigned char D4);

/*send PDO (index:1401h COB-ID (300h+node ID) used by RxPDO)*/

void SendCANPDO(unsigned char D1);

/*Send NMT (COB-ID : 0),Send NMT Error Control(COB-ID:700h + node ID)*/

void SendCANNMT(unsigned char RTR,unsigned char Cmd);

/*user define interrupt */

void UserCANInt(char CANInt)
{  

  if (CANInt) ;   /*avoid complier warning*/

  if (CANInt==0x01){ /*for Rx interrupt*/

  	L3On();
  	
	if(PrintFlag)
  ReceiveCANData();

  }
  else if (CANInt==0x02){ /*for Tx interrupt*/

  	L2On(); 

  }

  L2Off();

  L3Off();

}

void main(void)
{

		int ret;		

		unsigned short _channel=0,_hexValue=0,_choice=0,_quit=1;// if _quit value is 0,exit while loop

    InitLib();


    /********************************************************/

    /*                    Initial all LEDs                  */

    /********************************************************/

    L1Off();

    L2Off();

    L3Off();   

    /*******************************************************/

    /*      initialize and configure the CAN controller    */

    /*******************************************************/		

   ret=XC100Init(1,3,125000UL,0,0,0x00000000UL,0xffffffffUL);

    /* XCANInit function Parameter descriptions

       1            : for 7186EX(D) model number (7188XB(D) model number is 0)

       3            : for Receive and transmission interrupt

       125000UL     : for CAN baud

       0            : for BT0 of user defined baud

       0            : for BT1 of user defined baud

       0x00000000UL : for AccCode of CAN message filter

       0xffffffffUL : for AccMask of CAN message filter

    */

      switch (ret)
			{  

				/*Check if configuration is OK*/

		    case CAN_ResetError:

			    Print("Reset Error!\n\r");

			    return;

		    case CAN_SetACRError:

			    Print("\n\rSet ACR Error!");

			    return;

		    case CAN_SetAMRError:

			    Print("\n\rSet AMR Error!");

			    return;

		    case CAN_SetBaudRateError:

			    Print("\n\rSet Baud Rate Error!");

			    return;

		    case CAN_BaudNotSupport:

			    Print("\n\rBaud Rate Not Support!");

			    return;	

		    case CAN_ConfigError:

			    Print("\n\rConfiguration Failure!");

			    return;

		    default:

			    if (ret >0){

				    Print("Default=%d\n\r",ret);

				    return;

			    }

	    } //end switch
    
    /********************************************************/

    /*                     Enable Irq                       */

    /********************************************************/

    CAN_CreateBuffer(0,1000);   //Create RX Buffer

    CAN_CreateBuffer(1,1000);   //Create TX Buffer

    CAN_InstallIrq();//Set the interrupt function enable

    L1On(); /*Turn on LED1 for indicating that the demo is starting*/ 

		 while(_quit)
		 {

		 		DelayMs(500);
				PrintFlag = 0;
				Print("mode : (1)Change AO value (2)Change type code (0)exit :");
				Scanf("%d\n\r",&_choice);
				PrintFlag = 1;

	     switch(_choice)
				{

					case 1:

						//set SDO to change AO value (COB-ID form client to server (RxSDO),COB-ID:600h + node ID)
						PrintFlag = 0;
						Print("Choose a channel (1~4) which you want to write :");
						Scanf("%d\n\r",&_channel);												
						Print("Intput AO value (0xffff) : ");						
						Scanf("%x\n\r",&_hexValue);
						PrintFlag = 1;

						SendCANSDO(0x2B,0x11,0x64,(unsigned char)_channel,(unsigned char)_hexValue,(unsigned char)(_hexValue>>8),0x00,0x00);			

					break;

					case 2:

						//Send SDO to Change type code (COB-ID form client to server (RxSDO),COB-ID:600h + node ID)
						PrintFlag = 0;
						Print("Choose a channel (1~4) which you want to change the type code :");
						Scanf("%d\n\r",&_channel);				
						Print("%x\n\r",(unsigned char)_channel);										
						Print("intput type code (0x30~0x35) : ");
						Scanf("%x\n\r",&_hexValue);												
						PrintFlag = 1;

						SendCANSDO(0x2F,0x04,0x20,(unsigned char)_channel,(unsigned char)_hexValue,0x00,0x00,0x00);						

					break;			

					case 0:

						//send NMT to reset node
						SendCANNMT(0,0x81);	
						_quit=0;

					break;

					default :

						Print("Wrong choice,Please try angain\n\r");

					break;

				}//end switch

	}

	 L1Off();

	 L2Off();

	 L3Off();

	 CAN_Restore();	//free resource

}

void ReceiveCANData(void)
{

			unsigned char MsgData[8];

			unsigned char MsgMode,MsgRTR,MsgDataLen;

			unsigned long MsgID,MsgUpperTimeStamp,MsgLowerTimeStamp;

			int ret,i;			

			/***************************************************************/

      /*  Receive CAN messages  from receive buffer or from CAN bus  */

      /***************************************************************/

      /*

      	*MsgMode :This parameter is used for get the ID type (11-bit or 29-bit ID) of   

        					a CAN message

      	*MsgID: This is for obtaining the ID of a CAN message.                

				*RTR: This is for obtaining the RTR of a CAN message.

							RTR value  Meaning 

							   0			 This CAN message is not a remote transmit request message.

							 	 1			 This CAN message is a remote transmit request message.  

				*DataLen: This is for obtaining the data length of a CAN message.

				*Data: This is for obtaining the Data of a CAN message. The Data buffer 

							 size must be 8 bytes. 								 

				*UpperTime: Get the time stamp of a CAN message. The time stamp 

										unit is us (micro second), This parameter only show the 

										upper part of time stamp.   

					      		Real time stamp = upper part * 0x1000000UL+lower part 

				*LowerTime: Get the lower part of time stamp of a CAN message. 

      */

			ret=GetCANMsg(&MsgMode,&MsgID,&MsgRTR,&MsgDataLen,MsgData,&MsgUpperTimeStamp,&MsgLowerTimeStamp);

		

	    if (!ret)

	    {

	      MsgCnt++;

	      //Print received information

		    Print("%lu(%lus-%luus):Mode=%d,ID=%lx,RTR=%d,Len=%d",MsgCnt,MsgUpperTimeStamp

			   ,MsgLowerTimeStamp,MsgMode,MsgID,MsgRTR,MsgDataLen);

		    if (MsgDataLen && !MsgRTR)

		    {

			    Print(",Data=");

			    for (i=0;i<MsgDataLen;i++)

			    {

				    Print("%x,",MsgData[i]);

			    }

			    Print("\n\r");

			  	//Print only received data 

			  	//For TxSDO 

			  	if((MsgID >=0x580) && (MsgID<=0x5FF))

			  		{

			  			switch(MsgData[0])

			  			{

			  				case 0x4F:

			  					Print("Data Value = %02Xh\n\r",MsgData[4]);

			  				break;

			  				

			  				case 0x4B:			  					

			  					Print("Data Value = %02X%02XH\n\r",MsgData[5],MsgData[4]);

			  				break;

			  				

			  				case 0x47:

			  					Print("Data Value = %02X%02X%02Xh\n\r",MsgData[6],MsgData[5],MsgData[4]);

			  				break;		

			  					  					

			  				case 0x43:

			  					Print("Data Value = %02X%02X%02X%02Xh\n\r",MsgData[7],MsgData[6],MsgData[5],MsgData[4]);

			  				break;

			  			}

			  		}	

			  	  else if ((MsgID>=0x181 && MsgID<=0x18F)||(MsgID>=0x281 && MsgID<=0x28F)||(MsgID>=0x381 && MsgID<=0x38F)||(MsgID>=0x481 && MsgID<=0x48F))

			  		{//For TxPDO

			  			Print("Data value = ");

			  			for (i=0;i<MsgDataLen;i++)

			  			{

			  				if((MsgDataLen - (i+1))>0)

			  				{Print("%02Xh,",MsgData[i]);	}

			  				else

			  				{Print("%02Xh",MsgData[i]);	}

			  			}

			  			Print("\n\r");

			  		}

			  		else

			  			{

			  				;

			  			}

		    }

	    }

	    else

	    	{

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Reception Data Length Error!\n");

				    break;

			    case CAN_DataOverrun:

				    Print("Software Reception Data Buffer Overrun!\n");

				    break;

			    case CAN_ReceiveError:

				    Print("Receive data Error!\n");

				    break;

			    case CAN_ReceiveBufferEmpty: /*No message in receive buffer*/

				    break;

		    }

      }

}

void SendCANSDO(unsigned char cmd,unsigned char index_l,unsigned char index_h,unsigned char sub_index,

						    unsigned char D1,unsigned char D2,unsigned char D3,unsigned char D4)

{	

	int ret;

	MsgData[0]=cmd; 		//command specifier

	MsgData[1]=index_l; 		//index

	MsgData[2]=index_h; 		//index

	MsgData[3]=sub_index; 		//sub index 

	MsgData[4]=D1;//value 

	MsgData[5]=D2;

	MsgData[6]=D3;

	MsgData[7]=D4;

	ret=SendCANMsg(0,0x601,0,8,MsgData);	

	if (ret)

	 {

					    switch(ret)

					    {

						    case CAN_DataLengthError:

							    Print("Transmission Data Length Error!\n");

							    break;

						    case CAN_TransmitIncomplete:

							    Print("Transmission is incomplete!\n");

							    break;

						    case CAN_TransmitBufferLocked:

							    Print("CAN controller transmit Buffer is locked!\n");

							    break;

					    }

				

	 }

}

void SendCANPDO(unsigned char D1)

{

	int ret;

	//send PDO (COB-ID (200h+node ID) used by RxPDO)

	MsgData[0]=D1; //Do value

	MsgData[1]=0x00; 

	MsgData[2]=0x00; 

	MsgData[3]=0x00; 

	MsgData[4]=0x00; 

	MsgData[5]=0x00;

	MsgData[6]=0x00;

	MsgData[7]=0x00;

	ret=SendCANMsg(0,0x201,0,8,MsgData);	

	

	 if (ret)

	    {

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Transmission Data Length Error!\n");

				    break;

			    case CAN_TransmitIncomplete:

				    Print("Transmission is incomplete!\n");

				    break;

			    case CAN_TransmitBufferLocked:

				    Print("CAN controller transmit Buffer is locked!\n");

				    break;

		    }

		    //break;

	    }

}					    

void SendCANNMT(unsigned char RTR,unsigned char Cmd)

{

	int ret;

	if(RTR==0)

	{

			MsgData[0]=Cmd; //command specifier

			MsgData[1]=0x01; //node id

			MsgData[2]=0x00;

			MsgData[3]=0x00;

			MsgData[4]=0x00;

			MsgData[5]=0x00;

			MsgData[6]=0x00;

			MsgData[7]=0x00;

			ret=SendCANMsg(0,0x00,0,8,MsgData);						    

	}

	else

	{			

			MsgData[0]=0x00; //command specifier

			MsgData[1]=0x00; //node id

			MsgData[2]=0x00;

			MsgData[3]=0x00;

			MsgData[4]=0x00;

			MsgData[5]=0x00;

			MsgData[6]=0x00;

			MsgData[7]=0x00;

			ret=SendCANMsg(0,0x701,1,8,MsgData);						    		

	}

	 if (ret)

	    {

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Transmission Data Length Error!\n");

				    break;

			    case CAN_TransmitIncomplete:

				    Print("Transmission is incomplete!\n");

				    break;

			    case CAN_TransmitBufferLocked:

				    Print("CAN controller transmit Buffer is locked!\n");

				    break;

		    }

		    //break;

	    }

}